/* Sun Seeker
   group Cibus
   made for project Smart Environments of CreaTe M02
*/

#include <Servo.h>
Servo servo;
const int servoPin = 10; // Define the pin for the servo

const int ldrPin[] = {A2, A3, A4, A5}; // Define the pins for the LDR's

const int trigPin = 5; // Define the trigger pin for ultrasonic sensor
const int echoPin = 6; // Define the echo pin for ultrasonic sensor

// Define the pins for motor A
const int motorPWMa = 3;
const int motorDirA = 12;
const int motorBreakA = 9;

// Define the pins for motor B
const int motorPWMb = 11;
const int motorDirB = 13;
const int motorBreakB = 8;

int navCtrl = 0; // Variable used in the state machine. -1 means left, 1 means right, 0 means it executes the rest of the code
boolean seekLight = true; // Variable that decides if the it should steer towards or away from the sun
unsigned long time = millis();

void setup() {
  Serial.begin(9600);
  servo.attach(servoPin); // Attaches the servo object to the servoPin
  pinMode(trigPin, OUTPUT);
  pinMode(echoPin, INPUT);
  pinMode(motorDirA, OUTPUT) ;
  pinMode(motorBreakA, OUTPUT);
  pinMode(motorDirB, OUTPUT);
  pinMode(motorBreakB, OUTPUT);
}

void loop() {
  switch (navCtrl) {
    case (-1): // Drive backwards and turn left
    case (1): // Drive backwards and turn right
      if (millis() >= time + 1500) {
        if (millis() >= time + 3000) {
          navCtrl = 0;
        } else {
          if (navCtrl == -1) {
            left();
          } else {
            right();
          }
        }
      } else {
        backwards();
      }
      break;
    case (0):
      // Execute ultrasonic sensing
      int dist = ping_cm();
      delay(500);
      // if the utrasonic sensor senses an object closer than 20 cm compare the distance to the left and right of the object
      if (dist < 20 && dist != 0) {
        brake();
        obstacleCheck();
        time = millis();
      } else {
        navigation(getLDRs());
      }
      break;
  }
}

void obstacleCheck() {
  int pingLeft;
  int pingRight;
  servo.write(45);
  delay(500);
  pingLeft = ping_cm();
  delay(500);
  if (pingLeft == 0) {
    pingLeft = 1000;
  }
  servo.write(135);
  delay(500);
  pingRight = ping_cm();
  delay(500);
  if (pingRight == 0) {
    pingRight = 1000;
  }
  // Drive backwards and turn towards the highest side
  if (pingLeft >= pingRight) {
    navCtrl = -1;
  } else {
    navCtrl = 1;
  }
  servo.write(90); // move the servo back to face the front
  delay(500);
}

int ping_cm() {
  long duration;
  int distance;
  // Clears the trigPin
  digitalWrite(trigPin, LOW);
  delayMicroseconds(2);
  // Sets the trigPin on HIGH state for 10 micro seconds
  digitalWrite(trigPin, HIGH);
  delayMicroseconds(10);
  digitalWrite(trigPin, LOW);
  // Reads the echoPin, returns the sound wave travel time in microseconds
  duration = pulseIn(echoPin, HIGH);
  // Calculating the distance
  distance = duration * 0.034 / 2;
  Serial.println(distance);
  return distance;
}

int getLDRs() {
  int side[3];
  int highSide = 0;
  int highValue = 0;

  // Read the LDR values
  int sensorValue1 = analogRead(ldrPin[0]);
  int sensorValue2 = analogRead(ldrPin[1]);
  int sensorValue3 = analogRead(ldrPin[2]);
  int sensorValue4 = analogRead(ldrPin[3]);

  if (seekLight) {
    if (sensorValue1 < 1023 or sensorValue2 < 1023 or sensorValue3 < 1023 or sensorValue4 < 1023) {

      // If seekLight is true, calculate average of the front, left and right side and steer towards the highest value
      side[0] = (sensorValue1 + sensorValue2) / 2; // read average of the front side
      side[1] = (sensorValue2 + sensorValue3) / 2; // read average of the right side
      side[2] = (sensorValue4 + sensorValue1) / 2; // read average of the left side
    } else {
      // found sun
      brake();
      delay(10000); // 10 seconds
    }
  } else {
    if (sensorValue1 > 0 or sensorValue2 < 0 or sensorValue3 < 0 or sensorValue4 < 0) {

      // If seekLight is false, calculate average of the back, left and right side and steer away from the highest value
      side[0] = (sensorValue3 + sensorValue4) / 2; // read average of the back side
      side[1] = (sensorValue3 + sensorValue1) / 2; // read average of the right side
      side[2] = (sensorValue2 + sensorValue3) / 2; // read average of the left sidee
      Serial.println(side[0]);
    } else {
      // found shadow
      brake();
      delay(10000); // 10 seconds
    }
  }
  for (int i = 0; i < sizeof(side) / sizeof(int); i++) { // Get the side with the biggest light value
    if (side[i] > highValue) {
      highSide = i;
      highValue = side[i];
    }
  }
  return highSide;
}

void navigation(int highSide) {
  switch (highSide) { // drive towards or away from the side with the biggest light value
    case (0):
      forward();
      break;
    case (1):
      right();
      break;
    case (2):
      left();
      break;
  }
}

void forward() {
  digitalWrite(motorDirA, HIGH); //Establishes forward direction of Channel A
  digitalWrite(motorBreakA, LOW);   //Disengage the Brake for Channel A
  analogWrite(motorPWMa, 200);   //Spins the motor on Channel A at full speed

  digitalWrite(motorDirB, HIGH);  //Establishes forward direction of Channel B
  digitalWrite(motorBreakB, LOW);   //Disengage the Brake for Channel B
  analogWrite(motorPWMb, 255); //Spins the motor on Channel B at full speed
}

void backwards() {
  digitalWrite(motorDirA, LOW); //Establishes backward direction of Channel A
  digitalWrite(motorBreakA, LOW);   //Disengage the Brake for Channel A
  analogWrite(motorPWMa, 255);   //Spins the motor on Channel A at full speed

  digitalWrite(motorDirB, LOW);  //Establishes backward direction of Channel B
  digitalWrite(motorBreakB, LOW);   //Disengage the Brake for Channel B
  analogWrite(motorPWMb, 255); //Spins the motor on Channel B at full speed
}

void left() {
  digitalWrite(motorDirA, LOW); //Establishes forward direction of Channel A
  digitalWrite(motorBreakA, LOW);   //Disengage the Brake for Channel A
  analogWrite(motorPWMa, 255);   //Spins the motor on Channel A at full speed

  digitalWrite(motorDirB, HIGH);  //Establishes forward direction of Channel B
  digitalWrite(motorBreakB, LOW);   //Engage the Brake for Channel B
  analogWrite(motorPWMb, 255); //Spins the motor on Channel B at full speed
}

void right() {
  digitalWrite(motorDirA, HIGH); //Establishes forward direction of Channel A
  digitalWrite(motorBreakA, LOW);   //Engage the Brake for Channel A
  analogWrite(motorPWMa, 255);   //Spins the motor on Channel A at full speed

  digitalWrite(motorDirB, LOW);  //Establishes forward direction of Channel B
  digitalWrite(motorBreakB, LOW);   //Disengage the Brake for Channel B
  analogWrite(motorPWMb, 255); //Spins the motor on Channel B at half speed
}

void brake() {
  digitalWrite(motorDirA, HIGH); //Establishes forward direction of Channel A
  digitalWrite(motorBreakA, HIGH);   //Engage the Brake for Channel A
  analogWrite(motorPWMa, 255);   //Spins the motor on Channel A at full speed

  digitalWrite(motorDirB, HIGH);  //Establishes forward direction of Channel B
  digitalWrite(motorBreakB, HIGH);   //Engage the Brake for Channel B
  analogWrite(motorPWMb, 255); //Spins the motor on Channel B at half speed
}
