class Seeker {
  LDR[] ldr;
  Servo servo;
  Sonar sonar;
  PVector seekerPos, seekerMove;
  float seekerRot;
  int navCtrl, navRep;
  boolean seekLight; // Determines if the Sun Seeker drives towards light or away from it

  float seekerW, seekerH;
  Seeker(float _seekerX, float _seekerY) {
    seekerPos = new PVector(_seekerX, _seekerY);
    seekerMove = new PVector(0, 0);
    seekerRot = 0;
    navCtrl = 0;
    navRep = 0;
    seekerW = 60;
    seekerH = 100;
    seekLight = true;

    ldr = new LDR[4]; // The LDR's on the corners of the Sun Seeker. When facing up:
    ldr[0] = new LDR(-seekerW/2, -seekerH/2); // top left
    ldr[1] = new LDR(seekerW/2, -seekerH/2); // top right
    ldr[2] = new LDR(seekerW/2, seekerH/2); // bottom right
    ldr[3] = new LDR(-seekerW/2, seekerH/2); // bottom left

    servo = new Servo(0, -seekerH/2); // places the servo at the front of the Sun Seeker
    sonar = new Sonar(0, -seekerH/2); // places the ultrasonic sensor on top of the servo
  }

  void update() {
    for (int i = 0; i < ldr.length; i++) {
      ldr[i].update();
    }
    servo.update();
    sonar.update();

    switch(navCtrl) {
      case(-1): // move back then left
      case(1): // move back then right
      navRep += 1;
      if (navRep > 50) {
        if (navRep > 80) {
          navRep = 0;
          navCtrl = 0;
        } else {
          if (navCtrl == -1) {
            left();
          } else {
            right();
          }
        }
      } else {
        backward();
      }
      break;
      case(0): // main driving controller
      int dist = sonar.ping_cm(); // perform a sonar check
      if (dist < 100 && dist != 0) { // if the distance is smaller than 100:
        obstacleCheck();
      } else {
        navigate(getLDRs());
      }
    }
  }

  void display() {
    // Sun Seeker body
    if (seekLight) {
      fill(255);
    } else {
      fill(0);
    }
    pushMatrix();
    translate(seekerPos.x, seekerPos.y);
    rotate(seekerRot);
    rectMode(CENTER);
    noStroke();
    rect(0, seekerH*0.375, seekerW*1.5, seekerH*0.25);
    triangle(0, -seekerH/2, seekerW*0.75, seekerH*0.25, -seekerW*0.75, seekerH*0.25);
    stroke(150);
    rect(0, 0, seekerW, seekerH);
    popMatrix();

    for (int i = 0; i < ldr.length; i++) {
      ldr[i].display();
    }
    servo.display();
    sonar.display();
  }

  void obstacleCheck() {
    int pingLeft;
    int pingRight;
    servo.write(45); // check the distance 45 degrees to the left
    pingLeft = sonar.ping_cm();
    servo.write(135); // check the distance 45 degrees to the right
    pingRight = sonar.ping_cm();
    if (pingLeft > pingRight) {
      navCtrl = -1; // if the distance to the left is bigger than the distance to the right, drive back and then left
    } else {
      navCtrl = 1; // if the distance to the left is smaller than the distance to the right, drive back and then right
    }
    servo.write(90); // return the servo to face the front
  }

  int getLDRs() {
    int side[] = new int[3];
    int highSide = 0;
    int highValue = 0;
    if (seekLight) { // if it drives towards light:
      side[0] = (ldr[0].analogRead() + ldr[1].analogRead())/2; // read average of the front side
      side[1] = (ldr[1].analogRead() + ldr[2].analogRead())/2; // read average of the right side
      side[2] = (ldr[3].analogRead() + ldr[0].analogRead())/2; // read average of the left side
    } else { // if it drives away from light:
      side[0] = (ldr[2].analogRead() + ldr[3].analogRead())/2; // read average of the back side
      side[1] = (ldr[3].analogRead() + ldr[0].analogRead())/2; // read average of the right side
      side[2] = (ldr[1].analogRead() + ldr[2].analogRead())/2; // read average of the left side
    }
    for (int i = 0; i < side.length; i++) { // Get the side with the biggest light value
      if (side[i] > highValue) {
        highSide = i;
        highValue = side[i];
      }
    }
    // if the average across all LDRs is higher/lower than a value, the Sun Seeker will stop driving
    int ldrAvg = (ldr[0].analogRead()+ldr[1].analogRead()+ldr[2].analogRead()+ldr[3].analogRead())/4;
    if (seekLight) {
      if (ldrAvg > 900) {
        highSide = 3;
      }
    } else {
      if (ldrAvg < 500) {
        highSide = 3;
      }
    }
    return  highSide;
  }

  void navigate(int highSide) {
    switch (highSide) { // drive towards or away from the side with the biggest light value
      case(0):
      forward();
      break;
      case(1):
      right();
      break;
      case(2):
      left();
      break;
      case(3):
      brake();
      break;
    }
  }

  void forward() {
    seekerMove.set(0, -1);
    seekerMove.rotate(seekerRot);
    seekerPos.add(seekerMove);
  }

  void backward() {
    seekerMove.set(0, 1);
    seekerMove.rotate(seekerRot);
    seekerPos.add(seekerMove);
  }

  void left() {
    seekerRot -= PI/200;
  }

  void right() {
    seekerRot += PI/200;
  }

  void brake() {
    // execute code to make the Sun Seeker stop
  }
}
