// This class emulates an ultrasonic sensor by calculating the closest distance to any wall in a 30 degree radius
class Sonar extends Component {
  PVector[] rayCast;
  PVector ray;
  Sonar(float _compX, float _compY) {
    super(_compX, _compY);
    rayCast = new PVector[30];
    for (int i = 0; i < rayCast.length; i++) {
      rayCast[i] = new PVector(0, 0);
    }
    ray = new PVector(0, 0);
  }

  void display() {
    pushMatrix();
    translate(compScrPos.x, compScrPos.y);
    rotate(seeker.seekerRot+seeker.servo.servoRot);
    stroke(0, 0, 255);
    strokeWeight(2);
    line(-10, 0, 10, 0);
    popMatrix();

    ping_cm();
    noStroke();
    fill(255, 0, 0, 50);
    beginShape();
    vertex(compScrPos.x, compScrPos.y);
    arc(compScrPos.x, compScrPos.y, 200, 200, seeker.seekerRot+seeker.servo.servoRot-(15*PI)/180 - PI/2, seeker.seekerRot+seeker.servo.servoRot+(15*PI)/180 - PI/2);
    endShape(CLOSE);
  }

  int ping_cm() {
    // returns the closest distance to any wall in a 30 degree radius by doing a raycast
    int dist = 250;
    ray.set(0, -250);
    ray.rotate(seeker.seekerRot+seeker.servo.servoRot-(15*PI)/180);
    for (int n = 0; n < rayCast.length; n++) { // distributes the rays over a 30 degree radius
      rayCast[n] = ray.copy();
      rayCast[n].add(compScrPos);
      ray.rotate((PI)/180);
      for (int i = 0; i < walls.wall.length; i++) {
        for (int j = 0; j < walls.wall[i].length-1; j++) {
          // calculates the distance to any wall
          int distToIntersect = intersect(compScrPos.x, compScrPos.y, rayCast[n].x, rayCast[n].y, walls.wall[i][j].x, walls.wall[i][j].y, walls.wall[i][j+1].x, walls.wall[i][j+1].y);
          if (distToIntersect < dist) {
            dist = distToIntersect;
          }
        }
      }
    }
    return dist; // returns the smallest distance
  }

  int intersect(float x1, float y1, float x2, float y2, float x3, float y3, float x4, float y4) {
    // Calculates the distance from one point to an intersection between lines
    int distToIntersect = 250;
    float uA = ((x4-x3)*(y1-y3) - (y4-y3)*(x1-x3)) / ((y4-y3)*(x2-x1) - (x4-x3)*(y2-y1));
    float uB = ((x2-x1)*(y1-y3) - (y2-y1)*(x1-x3)) / ((y4-y3)*(x2-x1) - (x4-x3)*(y2-y1));
    if (uA >= 0 && uA <= 1 && uB >= 0 && uB <= 1) {
      float intersectionX = x1 + (uA * (x2-x1));
      float intersectionY = y1 + (uA * (y2-y1));
      distToIntersect = int(dist(compScrPos.x, compScrPos.y, intersectionX, intersectionY));
    }
    return distToIntersect; // returns the distance to the intersection
  }
}
