int w, h, cellSize; // grid size
int rx, ry; // robot
int gx, gy; // goal

int[][] grid;
int[][] nav;
boolean run;
int time = millis();

void setup() {
  size(800, 800, P2D);
  frameRate(5);
  surface.setResizable(true);
  init();
}

void init() {
  run = true;
  w = 20;
  h = 20;
  setCellSize();
  cellSize = width/w;
  if (h > w) cellSize = height/h; 
  rx = (int) random(w-1);
  ry = (int) random(h-1);
  gx = (int) random(w-1);
  gy = (int) random(h-1);

  grid = new int[h][w];
  nav = new int[h][w];

  createRoom();
  createNav();
}

void setCellSize() {
  cellSize = width/w;
  if (h > w || height < width) cellSize = height/h;
}

void draw() {
  if (time + 10000 < millis()) {
    init();
    time = millis();
  }

  background(0);
  setCellSize();
  drawGrid();

  // move robot to the goal
  if (nav[ry][rx] != 0) {
    move();
  }

  // draw player and move it
  noFill();
  stroke(0, 0, 255);
  strokeWeight(cellSize/16);
  ellipse(rx*cellSize+cellSize/2, ry*cellSize+cellSize/2, cellSize*.9, cellSize*.9);
}

void createRoom() {
  // setup grid
  for (int y=0; y<h; y++) {
    for (int x=0; x<w; x++) {
      grid[y][x] = 0;
      if ((y != ry || x != ry) && random(100) < 30) grid[y][x] = 1;
      /*if (y == 0 || x == 0 || y == h-1 || x == w-1) {
       grid[y][x] = 1;
       } else {
       grid[y][x] = 0;
       }*/
    }
  }
}

void createNav() {
  // setup nav
  for (int y=0; y<h; y++) {
    for (int x=0; x<w; x++) {
      nav[y][x] = -1;
      if (grid[y][x]==1) nav[y][x] = -2;
    }
  }

  // set goal
  nav[gy][gx] = 0;

  // while the grid isn't done
  while (run) {
    boolean update = false;

    // for every cell
    for (int y=0; y<h; y++) {
      for (int x=0; x<w; x++) {

        // get current value
        int c = nav[y][x];
        int newc = -1;

        int up = -2;
        int down = -2;
        int left = -2;
        int right = -2;

        // get surrounding cells
        if (y > 0) up = nav[y-1][x];
        if (y < h-1) down = nav[y+1][x];
        if (x > 0) left = nav[y][x-1];
        if (x < w-1) right = nav[y][x+1];

        // if the current cell has not been defined yet
        if (c == -1) {

          // add all nonnegative sides to an array
          int[] sides = {};
          if (up >= 0) sides = append(sides, up);
          if (down >= 0) sides = append(sides, down);
          if (left >= 0) sides = append(sides, left);
          if (right >= 0) sides = append(sides, right);

          // set the new c to the highest neighbour
          newc = max(up, down, max(left, right)); 

          // set the new c to the lowest neighbour
          if (sides.length > 0) {
            for (int i : sides) {
              newc = min(newc, i);
            }
            nav[y][x] = newc + 1; // set the current cell to the new value;
          }
        }

        // check if the cell has been updated
        if (nav[y][x] != c) update = true;
      }
    }

    // stop if nothing has been updated
    if (!update) run = false;
  }
}

void drawGrid() {
  // draw grid
  for (int y=0; y<h; y++) {
    for (int x=0; x<w; x++) {
      textAlign(CENTER, CENTER);
      rectMode(CORNER);
      noStroke();

      fill(0);
      if (nav[y][x] == -1) fill(150);
      if (nav[y][x] == 0) fill(150, 255, 150);
      if (nav[y][x] == -3) fill(150, 150, 255);
      if (nav[y][x] > 0) fill(255);

      rect(x*cellSize, y*cellSize, cellSize, cellSize);
      fill(255, 0, 0);
      textSize(cellSize/2);
      text(nav[y][x], x*cellSize+cellSize/2, y*cellSize+cellSize/2);
    }
  }
}

void move() {
  int c = nav[ry][rx];
  if (ry > 0 && nav[ry-1][rx] >= 0 && (nav[ry-1][rx] < c || c<=0)) {
    ry--;
  } else if (ry < h-1 && nav[ry+1][rx] >= 0 && (nav[ry+1][rx] < c || c<=0)) {
    ry++;
  } else if (rx > 0 && nav[ry][rx-1] >= 0 && (nav[ry][rx-1] < c || c<=0)) {
    rx--;
  } else if (rx < w-1 && nav[ry][rx+1] >= 0 && (nav[ry][rx+1] < c || c<=0)) {
    rx++;
  }
}

void keyPressed() {
  init();
}
